/**
 * Unicode character indicators.
 *
 * @module char-info/unicode
 * @preferred
 */ /** */
import { lookup } from "./internal/unicode-lookup";
import { BasicCharClassIndicator } from "./internal/indicators";
import { UnicodeCategory } from "./internal/names";
function homogenizeInputStr(str) {
    return str.toLowerCase().replace(/[_ -,]/g, "");
}
/**
 * Returns an indicator for characters belonging to the Unicode category,
 * `category`.
 * @param category The category to test for.
 */
export function uniInCategory(category) {
    category = homogenizeInputStr(category);
    if (category.length > 3) {
        category = lookup.longCategoryToCode.get(category);
    }
    let uGroup = lookup.categories.get(category);
    return new BasicCharClassIndicator(uGroup);
}
/**
 * Returns an indicator for characters belonging to a Unicode script, `script`.
 * @param script
 */
export function uniInScript(script) {
    script = homogenizeInputStr(script);
    let uGroup = lookup.scripts.get(script);
    return new BasicCharClassIndicator(uGroup);
}
/**
 * Returns an indicator for characters belonging to the Unicode block, `block`.
 * @param block
 */
export function uniInBlock(block) {
    block = homogenizeInputStr(block);
    let uGroup = lookup.blocks.get(block);
    return new BasicCharClassIndicator(uGroup);
}
/**
 * Indicator for Unicode decimal digit characters.
 */
export const uniIsDecimal = uniInCategory(UnicodeCategory.NumberDecimalDigit);
/**
 * Indicator for Unicode letters.
 */
export const uniIsLetter = uniInCategory(UnicodeCategory.Letter);
/**
 * Indicator for Unicode lowercase letters.
 */
export const uniIsLower = uniInCategory(UnicodeCategory.LetterLowercase);
/**
 * Indicator for Unicode uppercase letters.
 */
export const uniIsUpper = uniInCategory(UnicodeCategory.LetterUppercase);
/**
 * Indicator for Unicode inline spaces.
 */
export const uniIsSpace = uniInCategory(UnicodeCategory.SeparatorSpace);
/**
 * Indicator for Unicode vertical separators.
 */
export const uniIsNewline = uniInCategory(UnicodeCategory.Custom_SeparatorVertical);
/**
 * Returns the Unicode categories for a character or code.
 */
export const uniGetCategories = {
    code(code) {
        return lookup.allCategories.search(code, code);
    },
    char(char) {
        return uniGetCategories.code(char.charCodeAt(0));
    }
};
/**
 * Returns the Unicode scripts for a character or code.
 */
export const uniGetScripts = {
    code(code) {
        return lookup.allScripts.search(code, code);
    },
    char(char) {
        return uniGetScripts.code(char.charCodeAt(0));
    }
};
/**
 * Returns the Unicode block for a character or code.
 */
export const uniGetBlock = {
    code(code) {
        return lookup.allBlocks.search(code, code)[0];
    },
    char(char) {
        return uniGetBlock.code(char.charCodeAt(0));
    }
};
export { UnicodeScript, UnicodeCategory, UnicodeBlock } from "./internal/names";

//# sourceMappingURL=unicode.js.map